<?php
/**
 * Main Plugin
 *
 * @package   dorzki\WooCommerce\DynamicTaxes
 * @author    Dor Zuberi <admin@dorzki.io>
 * @copyright 2023 dorzki
 * @version   1.0.0
 */

namespace dorzki\WooCommerce\DynamicTaxes;

use WC_Cart;

// Block if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}


/**
 * Class Plugin
 *
 * @package dorzki\WooCommerce\DynamicTaxes
 */
class Plugin {
	/**
	 * Plugin constructor.
	 */
	public function __construct() {
		add_action( 'admin_init', [ $this, 'register_settings' ] );
		add_action( 'admin_menu', [ $this, 'register_settings_page' ] );

		add_action( 'woocommerce_cart_calculate_fees', [ $this, 'apply_dynamic_taxes' ] );
	}


	/**
	 * Register plugin settings.
	 *
	 * @return void
	 */
	public function register_settings(): void {
		register_setting( 'wc_dynamic_taxes', 'wc_dynamic_taxes' );

		// Register a new section.
		add_settings_section(
			'wc_dynamic_taxes_general',
			__( 'General', 'dorzki-wc-dynamic-taxes' ),
			'__return_null',
			'dorzki-wc-dynamic-taxes'
		);

		// Register plugin settings fields.
		add_settings_field(
			'wc_dynamic_taxes_name',
			__( 'Tax Name', 'dorzki-wc-dynamic-taxes' ),
			[ $this, 'settings_page_field_output' ],
			'dorzki-wc-dynamic-taxes',
			'wc_dynamic_taxes_general',
			[
				'type'      => 'text',
				'label_for' => 'wc_dynamic_taxes_name',
			]
		);

		add_settings_field(
			'wc_dynamic_taxes_amount',
			__( 'Tax Value', 'dorzki-wc-dynamic-taxes' ),
			[ $this, 'settings_page_field_output' ],
			'dorzki-wc-dynamic-taxes',
			'wc_dynamic_taxes_general',
			[
				'type'      => 'text',
				'label_for' => 'wc_dynamic_taxes_amount',
			]
		);

		$cats = get_terms(
			[
				'taxonomy'   => 'product_cat',
				'hide_empty' => false,
				'fields'     => 'id=>name',
			]
		);

		add_settings_field(
			'wc_dynamic_taxes_category',
			__( 'Apply to Category', 'dorzki-wc-dynamic-taxes' ),
			[ $this, 'settings_page_field_output' ],
			'dorzki-wc-dynamic-taxes',
			'wc_dynamic_taxes_general',
			[
				'type'      => 'select',
				'label_for' => 'wc_dynamic_taxes_category',
				'options'   => $cats,
			]
		);
	}

	/**
	 * Register new settings page menu item.
	 *
	 * @return void
	 */
	public function register_settings_page(): void {
		add_submenu_page(
			'woocommerce',
			__( 'Dynamic Taxes', 'dorzki-wc-dynamic-taxes' ),
			__( 'Dynamic Taxes', 'dorzki-wc-dynamic-taxes' ),
			'manage_options',
			'dorzki-wc-dynamic-taxes',
			[ $this, 'settings_page_output' ]
		);
	}


	/**
	 * Apply set dynamic taxes.
	 *
	 * @param WC_Cart $cart WooCommerce cart
	 *
	 * @return void
	 */
	public function apply_dynamic_taxes( WC_Cart $cart ): void {
		$tax_options = get_option( 'wc_dynamic_taxes' );

		if ( empty( $tax_options['wc_dynamic_taxes_category'] ) ) {
			return;
		}

		$total_taxes = 0;

		foreach ( $cart->get_cart_contents() as $item ) {
			$cats = wp_get_post_terms( $item['data']->get_ID(), 'product_cat', [ 'fields' => 'ids' ] );

			$total_taxes += ( in_array( (int) $tax_options['wc_dynamic_taxes_category'], $cats, true ) );
		}

		if ( $total_taxes ) {
			$cart->add_fee( sprintf( '%d&times; %s', $total_taxes, $tax_options['wc_dynamic_taxes_name'] ), $total_taxes * $tax_options['wc_dynamic_taxes_amount'], false );
		}
	}


	/**
	 * Output setting field.
	 *
	 * @param array $args Fields arguments
	 *
	 * @return void
	 */
	public function settings_page_field_output( array $args ): void {
		$allowed_types = [
			'select',
			'text',
			'tel',
			'email',
			'password',
			'date',
			'color',
			'search',
			'url',
		];

		if ( ! in_array( strtolower( $args['type'] ), $allowed_types, true ) ) {
			return;
		}

		// Get field value.
		$value = get_option( 'wc_dynamic_taxes' );

		if ( 'select' === $args['type'] ) {
			echo "<select name='wc_dynamic_taxes[{$args['label_for']}]'>"; // @phpcs:ignore
			echo '  <option value="">' . esc_html__( '---', 'dorzki-wc-dynamic-taxes' ) . '</option>';

			foreach ( $args['options'] as $id => $name ) {
				echo "  <option value='{$id}' " . selected( $value[ $args['label_for'] ], $id, true ) . ">{$name}</option>"; // @phpcs:ignore
			}

			echo '</select>';
		} else {
			echo "<input type='{$args['type']}' class='regular-text' name='wc_dynamic_taxes[{$args['label_for']}]' value='{$value[$args['label_for']]}'>"; // @phpcs:ignore
		}
	}

	/**
	 * Output settings page.
	 *
	 * @return void
	 */
	public function settings_page_output(): void {
		if ( ! current_user_can( 'manage_options' ) ) {
			return;
		}

		echo '<div class="wrap">';
		echo '	<h1>' . esc_html( get_admin_page_title() ) . '</h1>';
		echo '	<form action="options.php" method="post">';

		settings_fields( 'wc_dynamic_taxes' );

		do_settings_sections( 'dorzki-wc-dynamic-taxes' );

		submit_button( __( 'Save Settings', 'dorzki-wc-dynamic-taxes' ) );

		echo '	</form>';
		echo '</div>';
	}
}


// Init class.
new Plugin();
